/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 *
 *  Copyright (C) 2015 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  @(#) $Id: target_support.S 1203 2016-07-18 07:05:08Z ertl-honda $
 */

/*
 *  ターゲット依存モジュール アセンブリ言語部（ARM用）
 */

#define  TOPPERS_MACRO_ONLY
#define UINT_C(val)		(val)		/* uint_t型の定数を作るマクロ */
#define ULONG_C(val)	(val)		/* ulong_t型の定数を作るマクロ */
#include "kernel_impl.h"
#include "target_asm.inc"

#ifdef TOPPERS_SAFEG_SECURE

    ATEXT
    AALIGN(2)
    AGLOBAL(init_diag_reg)
ALABEL(init_diag_reg)
	mrc p15, 0, r0, c0, c0, 0	/* read main ID register */
	and r1, r0, #0x00f00000		/* variant */
	and r2, r0, #0x0000000f		/* revision */

	mrc p15, 0, r3, c15, c0, 1	/* read diagnostic register */

	/* ARM_ERRATA_742230 */
	cmp r2, #0x22		/*  up to r2p2 */
	orrle r3, r3, #1 << 4

	/* ARM_ERRATA_742231 */
	teq   r2, #0x20		/* in r2p0 */
	teqne r2, #0x21		/* in r2p1 */
	teqne r2, #0x22		/* in r2p2 */
	orreq r3, r3, #1 << 12
	orreq r3, r3, #1 << 22

	/* ARM_ERRATA_743622 */
	teq   r1, #0x00200000 /* in r2p */
	orreq	r3, r3, #1 << 6

	/* ARM_ERRATA_751472 */
	cmp r2, #0x30			/* prior to r3p0 */
	orrlt	r3, r3, #1 << 11

	/* ARM_ERRATA_794072 */
	orr	r3, r3, #1 << 4		@ set bit #4

	/* CONFIG_ARM_ERRATA_761320 */
	cmp	r2, #0x40			/* prior to r4p0 */
	orrlt	r3, r3, #1 << 21

	mcr p15, 0, r3, c15, c0, 1	/* write diagnostic register */
	bx	lr
#endif /* TOPPERS_SAFEG_SECURE */

